#include <stdio.h>
#include "transsecstoolwrapper.h"
#include "valueobject.h"
#include <chrono>
#include <thread>

transsecswrapper::TransSecsToolWrapper gem_native_wrapper  = transsecswrapper::TransSecsToolWrapper("GEMToolRuntime.jar");
bool temperatureAlarm = false;

// this simulates actual values read from a process PLC or other server
double RandomTemperatureValue() {
    //generate a random double between 50 and 62 (sometimes the value will be >60, this will trigger our test alarm)
    double r = static_cast <double> (rand()) / static_cast <double> (RAND_MAX); // generate a double between 0 and 1
    double testValue = 50.0 + r * 12.0;   
    return testValue;
}

// this simulates actual values read from a process PLC or other server
double RandomGasFlowValue() {
    //generate a random float between 8 and 9
    double r = static_cast <double> (rand()) / static_cast <double> (RAND_MAX); // generate a double between 0 and 1
    double testValue = 8.0 + r;
    return testValue;
}

/**Normally alarm triggers are connected to real PLC values. But for this demo we will
 * set an alarm if the temperature is >60 (and unset it when it is below 60).
 */
void CheckAlarms() {
    float temperature=RandomTemperatureValue();
    
    if (temperature>60.0) {
        //set the Alarm. This has the same effect as triggering with a boolean true if you were using a real data source.                
        if (!(temperatureAlarm)) {
            temperatureAlarm=true;
            gem_native_wrapper.Alarm("TemperatureProblem", temperatureAlarm);
        }
    }
    else {
        //clear the alarm. This has the same effect as triggering with a boolean false if you were using a real data source.
        if (temperatureAlarm) {
            temperatureAlarm=false;
            gem_native_wrapper.Alarm("TemperatureProblem", temperatureAlarm);
        }
    }
}
int main(int argc, char **argv) {    
    gem_native_wrapper.StartMain("GEMTool");
    
    int port = 5010;
    
    // persistance file saves GEM configuration between restarts (reports, enabled CEIDs, enabled Alarms, etc).
    gem_native_wrapper.Publish("gemtool/configuration/persistencefilename", "/tmp/testpersistence");
    gem_native_wrapper.Publish("gemtool/configuration/deviceid", 1);
    gem_native_wrapper.Publish("gemtool/configuration/port", port);
    
    gem_native_wrapper.Subscribe("gemtool/vids/controlstate", [](std::string, transsecswrapper::ValueObject value_object) {
        printf("Control state changed %s\n\r", value_object.GetStringValue().c_str());
    });

    
    gem_native_wrapper.Subscribe("gemtool/vids/ppid", [](std::string, transsecswrapper::ValueObject value_object) {
        //printf("Current time %li\n\r", value_object.GetLongValue());
        printf("New ppid %s\n\r", value_object.GetStringValue().c_str());
    });
    
    //note: online/offline and remote/local are usually set by a hardware switch on the tool 
    //  and read to get the actual values but here we set these values to defaults
    int online= 1; //online if set to 1 (offline when 0)
    int remote= 1; //remote if set to 1 (local when 0)
    // here we set the values of the VIDs OnlineOfflineState and LocalRemoteState
    // to some initial values (usually read from the tool hardware switches)
    gem_native_wrapper.Publish("OnlineOfflineState", online);//Go Online (still local)
    printf("Control State %s\n\r", gem_native_wrapper.Get("gemtool/vids/controlstate").GetStringValue().c_str() );
    gem_native_wrapper.Publish("LocalRemoteState", remote);//Allow Local-Remote transition
    printf("Control State %s\n\r", gem_native_wrapper.Get("gemtool/vids/ControlState").GetStringValue().c_str() );
    
    //gem_native_wrapper.Publish("EstablishCommunicationsTimeout", 5);//Allow Local-Remote transition
    //gem_native_wrapper.Publish("EnableSpooling", new LongValueObject(1));//Enable Spooling
    gem_native_wrapper.Publish("gemtool/vids/EnableSpooling", 0);//Disable Spooling
    
     //You can also change other GEM parameters to override the defaults set in the tool project
    // Recommendation is to limit MDLN and SoftRev to 6 chars  for backwards compatibility - here we demonstrate a longer version
    gem_native_wrapper.Publish("gemtool/vids/MDLN", "Model ABCDEFG");
    gem_native_wrapper.Publish("gemtool/vids/SoftRev","Rev1.0");//SoftRev
    gem_native_wrapper.Publish("gemtool/vids/ppid","InitialPPID");//set a ppid
    
    // We want to know when the host sends a Host Command.
    // So we set up a Message Notifier for this unsolicited message. 
    // Once this notifier is set up the code in the class "HostCommandNotifier"
    // will be called when the Host Command is received
    
    // create a notifier for the simple HostCommand message in the tool - gemtool/hostcommandstart/allparams
    gem_native_wrapper.Subscribe("gemtool/hostcommandstart",[](std::string topic, transsecswrapper::ValueObject value_object) {
        printf ("Host Command %s\n\r", value_object.GetStringValue().c_str());
        
        const char* response = "{\"HCACK\": { \"value\":\"4\", \"type\":10 },"
        "\"ErrorParamsList\": { \"values\": [  { \"value\":\"STEP\", \"type\":20 },"
        "{ \"value\":\"2\", \"type\":54 } ]  }}\n\r";
        
        //"Parm1List_0": { "values": [ "PPID","RecipeName" ] , "type":20 },
        gem_native_wrapper.PublishJson("gemtool/hostcommandreply/sendmessage", response);
    });
    
    //This next part runs the tool and sets random event and alarms once in awhile
    long waferCount = 1;
    long cycles = 0;
    srand (static_cast <unsigned> (time(0)));
    while (true) {
        
        //set some SVID values
        //gem_native_wrapper.Publish("gemtool/vids/ProcessTemperature",randomTemperatureValue());
        //gem_native_wrapper.Publish("gemtool/vids/GasFlow",randomGasFlowValue());
        std::this_thread::sleep_for(std::chrono::milliseconds(500));  
        cycles ++;
        
        //test event triggering every few wafer counts (events will not be triggered unless host has enabled them)
        
        if ((cycles % 10)==0) { //every 10 counts 
            
            waferCount++;
            gem_native_wrapper.Publish("gemtool/vids/WaferCount", waferCount);
            
            std::string ceid_trigger = std::string("{ \"values\":["
            "    { \"dvid\":\"WaferCount\", \"dvval\":\""+std::to_string(waferCount)+"\"  },"
            "    { \"dvid\":\"ADVID\",  \"dvval\":\"-2\"  }"
            " ] }");
            // the ceid triggers are in JSON format allowing DVIDs to be set.
            gem_native_wrapper.Trigger("Loaded", ceid_trigger.c_str());
        }
        
        //test alarms (alarms will not be triggered unless host has enabled them)
        CheckAlarms();
        
    }
}
