/*
 * Copyright 2020 <copyright holder> <email>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef TRANSSECSWRAPPER_VALUEOBJECT_H
# define TRANSSECSWRAPPER_VALUEOBJECT_H

# include <jni.h>
# include <list>
# include <string>

namespace transsecswrapper
{

    /**
     * @todo write docs
     */
    class ValueObject
    {
    public:
        /**
         * Default constructor
         */
        ValueObject(JavaVM* jvm, jobject object);
        /**
         * Default constructor
         */
        ValueObject(JNIEnv* env, jobject object);

        /**
         * Copy Constructor
         *
         * @param other
         */
        ValueObject ( const ValueObject& other );

        /**
         * Destructor
         */
        ~ValueObject();

        /**
         * Assign this value object's value to have the same value as other
         *
         * @param other
         * @return
         */
        ValueObject& operator= ( const ValueObject& other );

        /**
         * Compare the value objects, returning true iff the underlying java objects compare as equal. 
         *
         * @param other
         * @return
         */
        bool operator== ( const ValueObject& other ) const;

        /**
         * Returns !(==)
         *
         * @param other
         * @return
         */
        bool operator!= ( const ValueObject& other ) const;

        /** This will return the current value of the ValueObject as a boolean.
         * 
         * @return the value as a boolean
         */
        bool GetBoolValue();

        /** This will return the current value of the ValueObject as a char.
         * 
         * @return the value as a char
         */
        char GetByteValue();

        /** This will return the current value of the ValueObject as a short.
         * 
         * @return the value as a short
         */
        short GetShortValue();

        /** This will return the current value of the ValueObject as an int.
         * 
         * @return the value as a int
         */
        int GetIntValue();

        /** This will return the current value of the ValueObject as a long.
         * 
         * @return the value as a long
         */
        long GetLongValue();

        /** This will return the current value of the ValueObject as a float.
         * 
         * @return the value as a float
         */
        float GetFloatValue();

        /** This will return the current value of the ValueObject as a double.
         * 
         * @return the value as a double
         */
        double GetDoubleValue();

        /** This will return the current value of the ValueObject as a string.
         * 
         * @return the value as a string
         */
        std::string GetStringValue();

        /** This will return the age of the object in the number of milliseconds since 
         * Jan 1st 1970. */
        long GetTimeMillis();

        /** This will return the current value of the ValueObject as a list of value objects. This
         *  is most useful when the underlying value object represents a list, but for other value 
         *  objects it will return the value object wrapped in a list.
         * 
         * @return the value as a list
         */
        std::list<ValueObject> GetListValue();
    protected:
        jobject backing_object_;
        JavaVM* jvm;
    };

}
#endif                                                      // TRANSSECSWRAPPER_VALUEOBJECT_H
