
#include "../include/valueobject.h"
#include <stdexcept>
#include <iostream>

using namespace transsecswrapper;

ValueObject::ValueObject(JavaVM* jvm, jobject object)
{
    this->jvm = jvm;
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    backing_object_ = env->NewGlobalRef(object);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
}

transsecswrapper::ValueObject::ValueObject(JNIEnv* env, jobject object)
{
    backing_object_ = env->NewGlobalRef(object);
    int res = env->GetJavaVM(&jvm);
    if (res != 0) {
        throw std::runtime_error("Failed to get JVM from env");
    }
}


ValueObject::ValueObject ( const ValueObject& other )
{
    jvm = other.jvm;
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    backing_object_ = env->NewGlobalRef(other.backing_object_ );
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
}

ValueObject::~ValueObject()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    
    // We shouldn't throw in the destructor, so we just print to stderr
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            std::cerr << "Failed to AttachCurrentThread" << std::endl;
        }
    } else if (get_env_stat == JNI_EVERSION) {
            std::cerr << "GetEnv: version not supported" << std::endl;
    } else if (get_env_stat != JNI_OK) {
            std::cerr << "JNI error" << std::endl;
    }
    
    env->DeleteGlobalRef( backing_object_ );
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    backing_object_ = nullptr;
}

transsecswrapper::ValueObject & transsecswrapper::ValueObject::operator=(const transsecswrapper::ValueObject& other)
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    env->DeleteGlobalRef( backing_object_ );
    backing_object_ = env->NewGlobalRef(other.backing_object_ );
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    jvm = other.jvm;
}

bool transsecswrapper::ValueObject::operator!=(const transsecswrapper::ValueObject& other) const
{
  return !(*this == other);
}

bool transsecswrapper::ValueObject::operator==(const transsecswrapper::ValueObject& other) const
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "equals", "(Ljava/lang/Object;)Z");
    bool ret = env->CallBooleanMethod( backing_object_, mid, other.backing_object_ );
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

bool transsecswrapper::ValueObject::GetBoolValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getBoolValue", "()Z");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getBoolValue method not found");
    }
    bool ret = env->CallBooleanMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

char transsecswrapper::ValueObject::GetByteValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getByteValue", "()B");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getByteValue method not found");
    }
    char ret = env->CallByteMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

double transsecswrapper::ValueObject::GetDoubleValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getDoubleValue", "()D");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getDoubleValue method not found");
    }
    double ret = env->CallDoubleMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

float transsecswrapper::ValueObject::GetFloatValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getFloatValue", "()F");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getFloatValue method not found");
    }
    float ret = env->CallFloatMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

int transsecswrapper::ValueObject::GetIntValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getIntValue", "()I");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getIntValue method not found");
    }
    int ret = env->CallIntMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

long transsecswrapper::ValueObject::GetLongValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getLongValue", "()J");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getLongValue method not found");
    }
    long ret = env->CallLongMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

short transsecswrapper::ValueObject::GetShortValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getShortValue", "()S");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getShortValue method not found");
    }
    short ret = env->CallShortMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}


long transsecswrapper::ValueObject::GetTimeMillis()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getTimeMillis", "()J");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getTimeMillis method not found");
    }
    long ret = env->CallLongMethod( backing_object_, mid);
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret;
}

std::string transsecswrapper::ValueObject::GetStringValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jmethodID mid = env->GetMethodID(clazz, "getStringValue", "()Ljava/lang/String;");
    if (mid == NULL) {
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        throw std::runtime_error("getTimeMillis method not found");
    }
    jstring str = (jstring)env->CallObjectMethod( backing_object_, mid);
    const char* utf_string;
    jboolean is_copy;
    utf_string = env->GetStringUTFChars(str, &is_copy);
    std::string std_ret = std::string(utf_string);
    if (is_copy == JNI_TRUE) {
        env->ReleaseStringUTFChars(str, utf_string);
    }
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return std_ret;
}

std::list<ValueObject> transsecswrapper::ValueObject::GetListValue()
{
    JNIEnv* env;
    int get_env_stat = jvm->GetEnv((void **)&env, JNI_VERSION_1_8);
    bool is_detached = get_env_stat == JNI_EDETACHED;
    if (is_detached) {
        if (jvm->AttachCurrentThread((void **) &env, NULL) != 0) {
            throw std::runtime_error("Failed to AttachCurrentThread");
        }
    } else if (get_env_stat == JNI_EVERSION) {
            throw std::runtime_error("GetEnv: version not supported");
    } else if (get_env_stat != JNI_OK) {
        throw std::runtime_error("JNI error");
    }
    
    jclass clazz = env->GetObjectClass( backing_object_ );
    jclass clazz_class = env->GetObjectClass( clazz );
    
    // check if the class is an ArrayValueObject
    jmethodID get_class_name_mid = env->GetMethodID(clazz_class, "getName", "()Ljava/lang/String;");
    jstring str = (jstring)env->CallObjectMethod( clazz, get_class_name_mid);
    const char* utf_string;
    jboolean is_copy;
    utf_string = env->GetStringUTFChars(str, &is_copy);
    std::string class_name = std::string(utf_string);
    if (is_copy == JNI_TRUE) {
        env->ReleaseStringUTFChars(str, utf_string);
    }
    if (class_name != "com.ergotech.vib.valueobjects.ArrayValueObject") {
        // in this case, we should have a different value object
        if (is_detached) {
            jvm->DetachCurrentThread();
        }
        std::list<ValueObject> ret_list = std::list<ValueObject>();
        ret_list.push_back(ValueObject( *this));
        return ret_list;
    }
    
    jmethodID mid = env->GetMethodID(clazz, "getValues", "()Ljava/util/Vector;");
    std::list<ValueObject> ret_list = std::list<ValueObject>();
    jobject vector = env->CallObjectMethod( backing_object_, mid);
    jclass vector_clazz = env->GetObjectClass( vector );
    jmethodID size_mid = env->GetMethodID(vector_clazz, "size", "()I");
    jint vector_size = env->CallIntMethod(vector, size_mid);
    jmethodID get_mid = env->GetMethodID(vector_clazz, "get", "(I)Ljava/lang/Object;");
    for (int i = 0; i < vector_size; i++) {
        jobject object = env->CallObjectMethod(vector, get_mid, i);
        ret_list.push_back(ValueObject(jvm, object));
    }
    
    if (is_detached) {
        jvm->DetachCurrentThread();
    }
    return ret_list;
}


